import os, os.path
import sys
import time

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from sface.config import config
from sface.logwindow import LogWindow
from sface.rspecwindow import RSpecWindow
from sface.screens.configscreen import ConfigScreen
from sface.screens.mainscreen import MainScreen
from sface.screens.helpscreen import HelpScreen

# depending on the platform..
# could probably use Qt's resource system but looks overkill for just one file...
def locate_image_file (filename):
    for dir in [ '/usr/share', '/Applications/sface.app/Contents/Resources/sface' ] :
        for suffix in ['png','jpg']:
            attempt=os.path.join(dir,'images',"%s.%s"%(filename,suffix))
            if os.path.isfile(attempt) : return attempt
    return os.path.join('/could/not/locate/image/file',filename)
        

class Nav(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        
        self.title = QLabel("", self)
        scene=QGraphicsScene()
        pixmap = QPixmap(locate_image_file('graphic-sfa64'))
        logolabel=QLabel("",self)
        logolabel.setPixmap(pixmap)
        self.main = QLabel("", self)
        self.main.setAlignment(Qt.AlignRight)

        self.config = QLabel("", self)
        self.config.setAlignment(Qt.AlignRight)
        
        self.help = QLabel("", self)
        self.help.setAlignment(Qt.AlignRight)

        hlayout = QHBoxLayout()
        hlayout.addWidget(logolabel)
        hlayout.addWidget(self.title)
        hlayout.addStretch()
        gotolabel=QLabel("Go to: ", self)
        gotolabel.setAlignment(Qt.AlignRight)
        hlayout.addWidget(gotolabel)
        hlayout.addWidget(self.main)
        hlayout.addWidget(self.config)
        hlayout.addWidget(self.help)
        self.setLayout(hlayout)

    def setTitle(self, title):
        self.title.setText(title)

    def setLinks(self, main, config, help):
        self.main.setText(main)
        self.config.setText(config)
        self.help.setText(help)


class Status(QLabel):
    def __init__(self, parent=None):
        QLabel.__init__(self, "", parent)
        self.sliceUpdateDate()

    def set(self, msg, timeout):
        self.setText(msg)
        if timeout:
            QTimer.singleShot(timeout, self.reset)

    def sliceUpdateDate(self):
        rspec_file = config.getSliceRSpecFile()
        if not os.path.exists(rspec_file):
            return

        creation_time = os.stat(rspec_file).st_ctime
        last_update = time.ctime(creation_time)
        self.set("Slice data last updated on %s" % last_update, timeout=None)

    def reset(self):
        self.setText("")
        QTimer.singleShot(1500, self.sliceUpdateDate)


class MainWindow(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

        self.logWindow = LogWindow(self)
        self.rspecWindow = RSpecWindow(self)

        self.pix = QLabel(self)
        self.config_screen = ConfigScreen(self)
        self.main_screen = MainScreen(self)
        self.help_screen = HelpScreen(self)

        self.screens = QStackedWidget(self)
        self.screens.addWidget(self.main_screen)
        self.screens.addWidget(self.config_screen)
        self.screens.addWidget(self.help_screen)
        self.screens.addWidget(self.pix)
        self.next_screen = None

        self.nav = Nav(self)
        self.nav.setTitle(self.main_screen.getTitleText())
        self.nav.setLinks(self.main_screen.getLinkText(),
                          self.config_screen.getLinkText(),
                          self.help_screen.getLinkText())

        self.status = Status(self)
        self.log = QLabel("<a href='showlog'>Show Log</a>", self)
        self.rspec = QLabel("<a href='showlog'>Show RSpec</a>", self)

        hlayout = QHBoxLayout()
        hlayout.addWidget(self.status)
        hlayout.addStretch()
        hlayout.addWidget(self.rspec)
        hlayout.addWidget(self.log)

        layout = QVBoxLayout()
        layout.addWidget(self.nav)
        layout.addWidget(self.screens)
        layout.addLayout(hlayout)
        self.setLayout(layout)
        self.resize(800, 500)

        for link in (self.nav.main, self.nav.config, self.nav.help):
            self.connect(link, SIGNAL('linkActivated(QString)'),
                         self.animateToScreen)

        self.connect(self.log, SIGNAL('linkActivated(QString)'),
                     self.showLogWindow)
        self.connect(self.rspec, SIGNAL('linkActivated(QString)'),
                     self.showRSpecWindow)

    def redirectOutputToLog(self):
        self.logWindow.redirectOutput()

    def showLogWindow(self, link):
        self.logWindow.show()
        self.logWindow.resize(800, 200)
        self.logWindow.raise_()
        self.logWindow.activateWindow()

    def showRSpecWindow(self, link):
        self.rspecWindow.show()
        self.rspecWindow.resize(500, 640)
        self.rspecWindow.raise_()
        self.rspecWindow.activateWindow()


    def animatePixmap(self, y):
        self.pix.move(0, y)

    def animateToScreen(self, link):
        if link == self.config_screen.name:
            self.next_screen = self.config_screen
        elif link == self.main_screen.name:
            self.next_screen = self.main_screen
        elif link == self.help_screen.name:
            self.next_screen = self.help_screen

        curr_screen = self.screens.currentWidget()

        if self.next_screen == curr_screen:
            self.setStatus("Already showing %s" % curr_screen.getTitleText(), timeout=1000)
            return

        # This is an optimization to have a smoother animation. We
        # render the widget into a pixmap and animate that instead of
        # moving the whole widget around.
        pixmap = QPixmap(self.screens.size())
        curr_screen.render(pixmap)
        self.screens.setCurrentWidget(self.pix)
        self.pix.setPixmap(pixmap)

        timeLine = QTimeLine(500, self)
        timeLine.setFrameRange(0, self.screens.height());
        self.connect(timeLine, SIGNAL('frameChanged(int)'), self.animatePixmap)
        self.connect(timeLine, SIGNAL('finished()'), self.toNextScreen)
        timeLine.start()


    def toNextScreen(self):
        if self.next_screen == self.config_screen:
            self.toConfigScreen()
        elif self.next_screen == self.main_screen:
            self.toMainScreen()
        elif self.next_screen == self.help_screen:
            self.toHelpScreen()

    def toConfigScreen(self):
        self.screens.setCurrentWidget(self.config_screen)
        self.nav.setTitle(self.config_screen.getTitleText())

    def toHelpScreen(self):
        self.screens.setCurrentWidget(self.help_screen)
        self.nav.setTitle(self.help_screen.getTitleText())

    def toMainScreen(self):
        self.screens.setCurrentWidget(self.main_screen)
        self.nav.setTitle(self.main_screen.getTitleText())

    def setStatus(self, msg, timeout):
        self.status.set(msg, timeout)

    def nodeSelectionChanged(self, hostname):
        if self.rspecWindow.isVisible():
            self.rspecWindow.showNode(hostname)
